@@ -17,34 +17,40 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
+from collections import OrderedDict
 import datetime
 import logging
 import os
-import sys
+from unittest import mock
 
-import gobject
-import gtk
-import pango
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
+from gi.repository import GObject
+from gi.repository import Gtk
+from gi.repository import GtkSource
+from gi.repository import Pango
 
 from rednotebook.gui.menu import MainMenuBar
 from rednotebook.gui.options import OptionsManager
 from rednotebook.gui import customwidgets
 from rednotebook.gui.customwidgets import CustomComboBoxEntry, CustomListView
 from rednotebook.util import filesystem
+from rednotebook import info
 from rednotebook import templates
 from rednotebook.util import dates
-from rednotebook import undo
+from rednotebook.util import markup
+from rednotebook.util import utils
 from rednotebook.gui import categories
 from rednotebook.gui.exports import ExportAssistant
 from rednotebook.gui import browser
+from rednotebook.gui import browser_cef
 from rednotebook.gui import search
 from rednotebook.gui import editor
-from rednotebook.gui.clouds import Cloud
 from rednotebook.gui import insert_menu
 from rednotebook.gui import format_menu
 
 
-class MainWindow(object):
+class MainWindow:
     '''
     Class that holds the reference to the main glade file and handles
     all actions
@@ -53,33 +59,35 @@ class MainWindow(object):
 
         self.journal = journal
 
-        # Set the Glade file
+        # Load Glade file.
+        # TODO: Remove workaround for Windows once it is no longer needed.
         self.gladefile = os.path.join(filesystem.files_dir, 'main_window.glade')
-        self.builder = gtk.Builder()
-        self.builder.set_translation_domain('rednotebook')
-        try:
+        self.builder = Gtk.Builder()
+        # Register GtkSourceView so builder can use it when loading the file
+        # https://stackoverflow.com/q/10524196/434217
+        GObject.type_register(GtkSource.View)
+        if filesystem.IS_WIN:
+            import xml.etree.ElementTree as ET
+            tree = ET.parse(self.gladefile)
+            for node in tree.iter():
+                if 'translatable' in node.attrib:
+                    node.text = _(node.text)
+            xml_text = ET.tostring(tree.getroot(), encoding='unicode')
+            self.builder = Gtk.Builder.new_from_string(xml_text, len(xml_text))
+        else:
+            self.builder.set_translation_domain('rednotebook')
             self.builder.add_from_file(self.gladefile)
-        except gobject.GError, err:
-            logging.error('An error occured while loading the GUI: %s' % err)
-            logging.error('RedNotebook requires at least gtk+ 2.14. '
-                          'If you cannot update gtk, you might want to try an '
-                          'older version of RedNotebook.')
-            sys.exit(1)
 
         # Get the main window and set the icon
         self.main_frame = self.builder.get_object('main_frame')
         self.main_frame.set_title('RedNotebook')
-        try:
-            icon = gtk.gdk.pixbuf_new_from_file(os.path.join(filesystem.frame_icon_dir,
-                                                             'rednotebook.svg'))
-        except gobject.GError:
-            icon = gtk.gdk.pixbuf_new_from_file(os.path.join(filesystem.frame_icon_dir,
-                                                             'rn-192.png'))
+        icon = GdkPixbuf.Pixbuf.new_from_file(
+            os.path.join(filesystem.frame_icon_dir, 'rednotebook.svg'))
         self.main_frame.set_icon(icon)
 
         self.is_fullscreen = False
 
-        self.uimanager = gtk.UIManager()
+        self.uimanager = Gtk.UIManager()
 
         # Before fetching the menubar, add all menus and actiongroups.
         # Setup the toolbar items first to avoid warnings for missing actions.
@@ -88,14 +96,16 @@ class MainWindow(object):
         self.menubar_manager = MainMenuBar(self)
         self.menubar = self.menubar_manager.get_menu_bar()
         main_vbox = self.builder.get_object('vbox3')
-        main_vbox.pack_start(self.menubar, False)
+        main_vbox.pack_start(self.menubar, False, False, 0)
         main_vbox.reorder_child(self.menubar, 0)
 
-        self.undo_redo_manager = undo.UndoRedoManager(self)
+        self.undo_action = self.uimanager.get_action('/MainMenuBar/Edit/Undo')
+        self.redo_action = self.uimanager.get_action('/MainMenuBar/Edit/Redo')
 
-        self.calendar = Calendar(self.journal, self.builder.get_object('calendar'))
-        self.day_text_field = DayEditor(self.builder.get_object('day_text_view'),
-                                        self.undo_redo_manager)
+        self.calendar = MainCalendar(self.journal, self.builder.get_object('calendar'))
+        self.day_text_field = DayEditor(self.builder.get_object('day_text_view'))
+        self.day_text_field.connect('can-undo-redo-changed', self.update_undo_redo_buttons)
+        self.update_undo_redo_buttons()
         self.day_text_field.day_text_view.grab_focus()
         can_spell_check = self.day_text_field.can_spell_check()
         spell_check_enabled = bool(self.journal.config.read('spellcheck'))
@@ -121,31 +131,64 @@ class MainWindow(object):
         self.forward_one_day_button = self.builder.get_object('forward_one_day_button')
 
         self.edit_pane = self.builder.get_object('edit_pane')
-
-        self.html_editor = Preview(self.journal)
-        self.html_editor.webview.connect('button-press-event', self.on_browser_clicked)
-        self.html_editor.webview.connect('navigation-requested', self.on_browser_navigate)
-
         self.text_vbox = self.builder.get_object('text_vbox')
-        self.text_vbox.pack_start(self.html_editor)
+
+        if browser.WebKit2:
+            class Preview(browser.HtmlView):
+                def __init__(self, journal):
+                    browser.HtmlView.__init__(self)
+                    self.journal = journal
+                    self.internal = True
+
+                def show_day(self, new_day):
+                    html = self.journal.convert(new_day.text, 'xhtml')
+                    self.load_html(html)
+
+                def shutdown(self):
+                    pass
+
+            self.html_editor = Preview(self.journal)
+            self.html_editor.connect('button-press-event', self.on_browser_clicked)
+            self.html_editor.connect('decide-policy', self.on_browser_decide_policy)
+            self.text_vbox.pack_start(self.html_editor, True, True, 0)
+            self.html_editor.set_editable(False)
+        elif browser_cef.cef:
+            class Preview(browser_cef.HtmlView):
+                def __init__(self, journal):
+                    super().__init__()
+                    self.journal = journal
+                    self.internal = True
+
+                def show_day(self, new_day):
+                    html = self.journal.convert(new_day.text, 'xhtml')
+                    self.load_html(html)
+
+                def highlight(self, text):
+                    pass
+
+            self.html_editor = Preview(self.journal)
+            self.text_vbox.pack_start(self.html_editor, True, True, 0)
+        else:
+            self.html_editor = mock.MagicMock()
+            self.html_editor.internal = False
+            preview_button = self.builder.get_object('preview_button')
+            preview_button.set_label(_('Preview in Browser'))
+
         self.html_editor.hide()
-        self.html_editor.set_editable(False)
+
         self.preview_mode = False
 
         # Let the edit_paned respect its childs size requests
         self.edit_pane.child_set_property(self.text_vbox, 'shrink', False)
 
         # Add InfoBar.
-        if customwidgets.Info:
-            self.infobar = customwidgets.Info()
-            self.text_vbox.pack_start(self.infobar, False, False)
-            self.text_vbox.reorder_child(self.infobar, 1)
-        else:
-            self.infobar = self.statusbar
+        self.infobar = customwidgets.Info()
+        self.text_vbox.pack_start(self.infobar, False, False, 0)
+        self.text_vbox.reorder_child(self.infobar, 1)
 
         # Add TemplateBar.
         self.template_bar = customwidgets.TemplateBar()
-        self.text_vbox.pack_start(self.template_bar, False, False)
+        self.text_vbox.pack_start(self.template_bar, False, False, 0)
         self.text_vbox.reorder_child(self.template_bar, 1)
         self.template_bar.hide()
 
@@ -191,6 +234,10 @@ class MainWindow(object):
         self.set_tooltips()
         self.setup_tray_icon()
 
+        # Show/hide the "tags" panel on the right.
+        self.builder.get_object('annotations_pane').set_visible(
+            self.journal.config.read('showTagsPane'))
+
     def set_tooltips(self):
         '''
         Little work-around:
@@ -222,24 +269,25 @@ class MainWindow(object):
             (self.forward_one_day_button, 'clicked', '<Ctrl>Page_Down'),
         ]
         for button, signal, shortcut in shortcuts:
-            (keyval, mod) = gtk.accelerator_parse(shortcut)
+            (keyval, mod) = Gtk.accelerator_parse(shortcut)
             button.add_accelerator(signal, self.accel_group,
-                                   keyval, mod, gtk.ACCEL_VISIBLE)
+                                   keyval, mod, Gtk.AccelFlags.VISIBLE)
 
     def _on_key_press_event(self, widget, event):
         # Exit fullscreen mode with ESC.
-        if event.keyval == gtk.keysyms.Escape and self.is_fullscreen:
+        if event.keyval == Gdk.KEY_Escape and self.is_fullscreen:
             self.toggle_fullscreen()
 
     # TRAY-ICON / CLOSE --------------------------------------------------------
 
     def setup_tray_icon(self):
-        self.tray_icon = gtk.StatusIcon()
+        self.tray_icon = Gtk.StatusIcon()
+        self.tray_icon.set_name('RedNotebook')
         visible = (self.journal.config.read('closeToTray') == 1)
         self.tray_icon.set_visible(visible)
         logging.debug('Tray icon visible: %s' % visible)
 
-        self.tray_icon.set_tooltip('RedNotebook')
+        self.tray_icon.set_tooltip_text('RedNotebook')
         # TODO: Try using the svg here as well.
         icon_file = os.path.join(self.journal.dirs.frame_icon_dir, 'rn-32.png')
         self.tray_icon.set_from_file(icon_file)
@@ -267,13 +315,13 @@ class MainWindow(object):
         </ui>'''
 
         # Create an ActionGroup
-        actiongroup = gtk.ActionGroup('TrayActionGroup')
+        actiongroup = Gtk.ActionGroup('TrayActionGroup')
 
         # Create actions
         actiongroup.add_actions([
-            ('Show', gtk.STOCK_MEDIA_PLAY, _('Show RedNotebook'),
+            ('Show', Gtk.STOCK_MEDIA_PLAY, _('Show RedNotebook'),
                 None, None, lambda widget: self.show()),
-            ('Quit', gtk.STOCK_QUIT, None, None, None, self.on_quit_activate),
+            ('Quit', Gtk.STOCK_QUIT, None, None, None, self.on_quit_activate),
         ])
 
         # Add the actiongroup to the uimanager
@@ -286,7 +334,7 @@ class MainWindow(object):
         menu = self.uimanager.get_widget('/TrayMenu')
 
         menu.popup(
-            None, None, gtk.status_icon_position_menu, button,
+            None, None, Gtk.status_icon_position_menu, button,
             activate_time, status_icon)
 
     def show(self):
@@ -307,6 +355,7 @@ class MainWindow(object):
         if self.journal.config.read('closeToTray'):
             self.hide()
         else:
+            self.html_editor.shutdown()
             self.journal.exit()
 
         # We never call the default handler. Otherwise, the window would be
@@ -329,8 +378,8 @@ class MainWindow(object):
         columns = [('1', str), ('2', str)]
         overall_list = CustomListView(columns)
         day_list = CustomListView(columns)
-        overall_box.pack_start(overall_list, True, True)
-        day_box.pack_start(day_list, True, True)
+        overall_box.pack_start(overall_list, True, True, 0)
+        day_box.pack_start(day_list, True, True, 0)
         setattr(self.stats_dialog, 'overall_list', overall_list)
         setattr(self.stats_dialog, 'day_list', day_list)
         for list in [overall_list, day_list]:
@@ -343,7 +392,7 @@ class MainWindow(object):
         edit_button = self.builder.get_object('edit_button')
         preview_button = self.builder.get_object('preview_button')
 
-        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        size_group = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
         size_group.add_widget(edit_button)
         size_group.add_widget(preview_button)
 
@@ -354,6 +403,8 @@ class MainWindow(object):
 
             edit_button.show()
             preview_button.hide()
+
+            self.disable_undo_redo_buttons()
         else:
             # Enter edit mode
             edit_scroll.show()
@@ -362,11 +413,19 @@ class MainWindow(object):
             preview_button.show()
             edit_button.hide()
 
+            self.update_undo_redo_buttons()
+
+        # Interacting with the CEF browser makes the main window inactive, so
+        # we make it active again.
+        self.main_frame.present()
+
         self.template_manager.set_template_menu_sensitive(not preview)
         self.insert_actiongroup.set_sensitive(not preview)
         self.format_actiongroup.set_sensitive(not preview)
         self.insert_button.set_sensitive(not preview)
         self.format_button.set_sensitive(not preview)
+        for action in ['Cut', 'Paste']:
+            self.uimanager.get_widget('/MainMenuBar/Edit/%s' % action).set_sensitive(not preview)
 
         self.preview_mode = preview
 
@@ -375,34 +434,57 @@ class MainWindow(object):
         self.change_mode(preview=False)
         # Select (not only highlight) previously selected text by giving focus
         # to the day editor.
-        gobject.idle_add(self.day_text_field.day_text_view.grab_focus)
+        GObject.idle_add(self.day_text_field.day_text_view.grab_focus)
 
     def on_preview_button_clicked(self, button):
         self.journal.save_old_day()
-        self.html_editor.show_day(self.day)
-        self.change_mode(preview=True)
+        if self.html_editor.internal:
+            self.html_editor.show_day(self.day)
+            self.change_mode(preview=True)
+        else:
+            date_format = self.journal.config.read('exportDateFormat')
+            date_string = dates.format_date(date_format, self.day.date)
+            markup_string = markup.get_markup_for_day(self.day)
+            html = self.journal.convert(
+                markup_string, 'xhtml',
+                headers=[date_string + ' - RedNotebook', '', ''],
+                options={'toc': 0})
+            utils.show_html_in_browser(
+                html, os.path.join(self.journal.dirs.temp_dir, 'day.html'))
 
     def on_browser_clicked(self, webview, event):
-        # Double-click -> Change to edit mode.
-        if event.type == gtk.gdk._2BUTTON_PRESS:
+        if event.type == Gdk.EventType._2BUTTON_PRESS:
+            # Double-click -> Change to edit mode.
             self.change_mode(preview=False)
-            # Stop processing that event
+            # Stop processing this event.
+            return True
+        elif event.button == 3:
+            # Right-click -> don't show context menu.
             return True
 
     # ----------------------------------------------------------- MODE-SWITCHING
 
     def setup_search(self):
-        self.search_tree_view = search.SearchTreeView(self)
+        always_show_results = not browser.WebKit2
+        self.search_tree_view = search.SearchTreeView(self, always_show_results)
         self.search_tree_view.show()
-        scroll = gtk.ScrolledWindow()
-        scroll.add(self.search_tree_view)
-        self.builder.get_object('search_container').pack_start(scroll)
-        self.search_box = search.SearchComboBox(
-            self.builder.get_object('search_box'), self)
+        self.search_scroll = Gtk.ScrolledWindow()
+        if always_show_results:
+            self.search_scroll.show()
+        self.search_scroll.add(self.search_tree_view)
+        self.search_box = search.SearchComboBox(Gtk.ComboBox.new_with_entry(), self)
+        self.search_box.combo_box.show()
+        search_container = self.builder.get_object('search_container')
+        search_container.pack_start(self.search_box.combo_box, False, False, 0)
+        search_container.pack_start(self.search_scroll, True, True, 0)
 
     def setup_clouds(self):
-        self.cloud = Cloud(self.journal)
-        self.builder.get_object('search_container').pack_start(self.cloud)
+        if browser.WebKit2:
+            from rednotebook.gui import clouds
+            self.cloud = clouds.Cloud(self.journal)
+            self.builder.get_object('search_container').pack_end(self.cloud, True, True, 0)
+        else:
+            self.cloud = mock.MagicMock()
 
     def on_main_frame_configure_event(self, widget, event):
         '''
@@ -422,8 +504,8 @@ class MainWindow(object):
         minimized, maximized, made sticky, made not sticky, shaded or
         unshaded.
         '''
-        if event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED:
-            maximized = bool(event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
+        if event.changed_mask & Gdk.WindowState.MAXIMIZED:
+            maximized = bool(event.new_window_state & Gdk.WindowState.MAXIMIZED)
             self.journal.config['mainFrameMaximized'] = int(maximized)
 
     def toggle_fullscreen(self):
@@ -444,19 +526,20 @@ class MainWindow(object):
     def on_forward_one_day_button_clicked(self, widget):
         self.journal.go_to_next_day()
 
-    def on_browser_navigate(self, webview, frame, request):
+    def on_browser_decide_policy(self, webview, decision, decision_type):
         '''
-        We want to load files and links externally
+        We want to load files and links externally.
         '''
-        if self.html_editor.loading_html:
-            # Keep processing
-            return False
+        if decision_type == browser.WebKit2.PolicyDecisionType.NAVIGATION_ACTION:
+            action = decision.get_navigation_action()
+            if action.is_user_gesture():
+                uri = action.get_request().get_uri()
+                logging.info('Clicked URI "%s"' % uri)
+                filesystem.open_url(uri)
 
-        uri = request.get_uri()
-        logging.info('Clicked URI "%s"' % uri)
-        filesystem.open_url(uri)
+                decision.ignore()
 
-        # Stop processing that event
+        # Stop processing this event.
         return True
 
     def get_new_journal_dir(self, title, message):
@@ -465,18 +548,18 @@ class MainWindow(object):
         label = self.builder.get_object('dir_chooser_label')
 
         label.set_markup('<b>' + message + '</b>')
-        dir_chooser.set_current_folder(self.journal.dirs.data_dir)
+        dir_chooser.set_current_folder(os.path.dirname(self.journal.dirs.data_dir))
 
         response = dir_chooser.run()
         # Retrieve the dir now, because it will be cleared by the call to hide().
         new_dir = dir_chooser.get_filename()
         dir_chooser.hide()
 
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             if new_dir is None:
                 self.journal.show_message(_('No directory selected.'), error=True)
                 return None
-            return new_dir.decode('utf-8')
+            return new_dir
         return None
 
     def show_save_error_dialog(self, exit_imminent):
@@ -492,12 +575,12 @@ class MainWindow(object):
         answer = dialog.run()
         dialog.hide()
 
-        if answer == gtk.RESPONSE_OK:
+        if answer == Gtk.ResponseType.OK:
             # Even if the user aborts the Save-As dialog, we don't want to exit.
             self.journal.is_allowed_to_exit = False
             # Let the user select a new directory. Nothing has been saved yet.
             self.menubar_manager.on_save_as_menu_item_activate(None)
-        elif answer == gtk.RESPONSE_CANCEL and exit_imminent:
+        elif answer == Gtk.ResponseType.CANCEL and exit_imminent:
             self.journal.is_allowed_to_exit = False
         # Do nothing if user wants to exit without saving
 
@@ -515,18 +598,13 @@ class MainWindow(object):
         # Remember window position
         config['mainFrameX'], config['mainFrameY'] = self.main_frame.get_position()
 
-        # Actually this is unnecessary as the list gets saved when it changes
-        # so we use it to sort the list ;)
-        config.write_list('cloudIgnoreList', sorted(self.cloud.ignore_list))
-        config.write_list('cloudIncludeList', sorted(self.cloud.include_list))
-
     def load_values_from_config(self):
         config = self.journal.config
         main_frame_width = config.read('mainFrameWidth')
         main_frame_height = config.read('mainFrameHeight')
 
-        screen_width = gtk.gdk.screen_width()
-        screen_height = gtk.gdk.screen_height()
+        screen_width = Gdk.Screen.width()
+        screen_height = Gdk.Screen.height()
 
         main_frame_width = min(main_frame_width, screen_width)
         main_frame_height = min(main_frame_height, screen_height)
@@ -545,10 +623,10 @@ class MainWindow(object):
                 if 0 <= x <= screen_width and 0 <= y <= screen_height:
                     self.main_frame.move(x, y)
                 else:
-                    self.main_frame.set_position(gtk.WIN_POS_CENTER)
+                    self.main_frame.set_position(Gtk.WindowPosition.CENTER)
             except (ValueError, TypeError):
                 # Values have not been set or are not valid integers
-                self.main_frame.set_position(gtk.WIN_POS_CENTER)
+                self.main_frame.set_position(Gtk.WindowPosition.CENTER)
 
         self.builder.get_object('main_pane').set_position(config.read('leftDividerPosition'))
         # By default do not show tags pane.
@@ -558,16 +636,16 @@ class MainWindow(object):
 
     def set_font(self, font_name):
         self.day_text_field.set_font(font_name)
-        self.html_editor.set_font_size(pango.FontDescription(font_name).get_size() / pango.SCALE)
+        self.html_editor.set_font_size(Pango.FontDescription(font_name).get_size() / Pango.SCALE)
 
     def setup_template_menu(self):
         def update_menu(button):
             self.template_button.set_menu(self.template_manager.get_menu())
 
         self.template_button = customwidgets.ToolbarMenuButton(
-            gtk.STOCK_PASTE, self.template_manager.get_menu())
+            Gtk.STOCK_PASTE, self.template_manager.get_menu())
         self.template_button.set_label(_('Template'))
-        self.template_button.connect('show-menu', update_menu)
+        self.template_button.connect('clicked', update_menu)
         self.template_button.set_tooltip_text(_(
             "Insert this weekday's template. "
             "Click the arrow on the right for more options"))
@@ -592,7 +670,7 @@ class MainWindow(object):
         self.day_text_field.show_day(day)
 
         # Only switch mode automatically if set in preferences.
-        if self.journal.config.get('autoSwitchMode', 1):
+        if self.journal.config.read('autoSwitchMode') and self.html_editor.internal:
             if day.has_text and not self.preview_mode:
                 self.change_mode(preview=True)
             elif not day.has_text and self.preview_mode:
@@ -603,7 +681,6 @@ class MainWindow(object):
             self.html_editor.show_day(day)
 
         self.categories_tree_view.set_day_content(day)
-        self.undo_redo_manager.set_stack(new_date)
 
     def get_day_text(self):
         return self.day_text_field.get_text()
@@ -613,78 +690,111 @@ class MainWindow(object):
         self.day_text_field.highlight(search_text)
 
     def show_message(self, title, msg, msg_type):
-        if msg_type == gtk.MESSAGE_ERROR:
+        if msg_type == Gtk.MessageType.ERROR:
             self.infobar.show_message(title, msg, msg_type)
         else:
             self.statusbar.show_message(title, msg, msg_type)
 
+    def disable_undo_redo_buttons(self):
+        self.undo_action.set_sensitive(False)
+        self.redo_action.set_sensitive(False)
 
-class Preview(browser.HtmlView):
-    def __init__(self, journal, *args, **kwargs):
-        browser.HtmlView.__init__(self, *args, **kwargs)
-        self.journal = journal
-        self.day = None
+    def update_undo_redo_buttons(self, _gobject=None):
+        """Enable/disable undo+redo actions according to the current text buffer
 
-    def show_day(self, new_day):
-        # Save the position in the preview pane for the old day
-        if self.day:
-            self.day.last_preview_pos = (self.get_hscrollbar().get_value(),
-                                         self.get_vscrollbar().get_value())
-
-        # Show new day
-        self.day = new_day
-        html = self.journal.convert(self.day.text, 'xhtml')
-        self.load_html(html)
-
-        if self.day.last_preview_pos is not None:
-            x, y = self.day.last_preview_pos
-            gobject.idle_add(self.get_hscrollbar().set_value, x)
-            gobject.idle_add(self.get_vscrollbar().set_value, y)
+        The _gobject parameter is unused, but it's necessary for the method
+        to be connected to a GObject signal.
+        """
+        can_undo = self.day_text_field.day_text_buffer.can_undo()
+        self.undo_action.set_sensitive(can_undo)
+        can_redo = self.day_text_field.day_text_buffer.can_redo()
+        self.redo_action.set_sensitive(can_redo)
 
 
 class DayEditor(editor.Editor):
+    n_recent_buffers = 10  # How many recent buffers to store
+    _t2t_highlighting = None
+    _style_scheme = None
+
     def __init__(self, *args, **kwargs):
         editor.Editor.__init__(self, *args, **kwargs)
         self.day = None
-        self.scrolled_win = self.day_text_view.get_parent()
+        # Store buffers for recently edited days - these preserve undo history
+        # and cursor position. Once a buffer drops out of this, it needs to be
+        # recreated: at this point, the cursor and undo are lost.
+        self.recent_buffers = OrderedDict()
+
+    def _get_t2t_highlighting(self):
+        if self._t2t_highlighting is None:
+            # Load our own copy of t2t syntax highlighting
+            lm = GtkSource.LanguageManager.get_default()
+            search_path = lm.get_search_path()
+            if filesystem.files_dir not in search_path:
+                search_path.insert(0, filesystem.files_dir)
+                lm.set_search_path(search_path)
+            self._t2t_highlighting = lm.get_language('t2t')
+        return self._t2t_highlighting
+
+    def _get_style_scheme(self):
+        if self._style_scheme is None:
+            # Load our customised variant of the Tango scheme
+            sm = GtkSource.StyleSchemeManager.get_default()
+            if filesystem.files_dir not in sm.get_search_path():
+                sm.prepend_search_path(filesystem.files_dir)
+            self._style_scheme = sm.get_scheme('rednotebook')
+        return self._style_scheme
+
+    def _get_buffer(self, key, text):
+        '''Get an editing buffer for a given item
+
+        If key is in our cache of recently used buffers, its buffer is retrieved
+        and text is ignored. Otherwise, a new buffer is constructed with text.
+        '''
+        if key in self.recent_buffers:
+            self.recent_buffers.move_to_end(key)
+            return self.recent_buffers[key]
+
+        buf = self.recent_buffers[key] = GtkSource.Buffer.new()
+        buf.set_style_scheme(self._get_style_scheme())
+        buf.set_language(self._get_t2t_highlighting())
+        buf.create_tag('highlighter', background='Yellow')
+        buf.begin_not_undoable_action()
+        buf.set_text(text)
+        buf.end_not_undoable_action()
+
+        if len(self.recent_buffers) > self.n_recent_buffers:
+            self.recent_buffers.popitem(last=False)
+
+        # Only one buffer is added at a time, so the 'if' above should always
+        # keep us at most n_recent_buffers. If code is added to add to the list
+        # elsewhere, it should check the maximum length as well.
+        assert len(self.recent_buffers) <= self.n_recent_buffers
+
+        return buf
+
+    def _get_buffer_for_day(self, day):
+        return self._get_buffer(day.date, day.text)
 
     def show_day(self, new_day):
-        # Save the position in the edit pane for the old day
-        if self.day:
-            cursor_pos = self.day_text_buffer.get_property('cursor-position')
-            # If there is a selection we save it, else we save the cursor position
-            selection = self.day_text_buffer.get_selection_bounds()
-            if selection:
-                selection = [it.get_offset() for it in selection]
-            else:
-                selection = [cursor_pos, cursor_pos]
-            self.day.last_edit_pos = (self.scrolled_win.get_hscrollbar().get_value(),
-                                      self.scrolled_win.get_vscrollbar().get_value(),
-                                      selection)
-
         # Show new day
         self.day = new_day
-        self.set_text(self.day.text, undoing=True)
+        buf = self._get_buffer_for_day(new_day)
+        self.replace_buffer(buf)
+        self.day_text_view.grab_focus()
 
         if self.search_text:
             # If a search is currently made, scroll to the text and return.
-            gobject.idle_add(self.scroll_to_text, self.search_text)
+            GObject.idle_add(self.scroll_to_text, self.search_text)
+            GObject.idle_add(self.highlight, self.search_text)
             return
 
-        if self.day.last_edit_pos is not None:
-            x, y, selection = self.day.last_edit_pos
-            gobject.idle_add(self.scrolled_win.get_hscrollbar().set_value, x)
-            gobject.idle_add(self.scrolled_win.get_vscrollbar().set_value, y)
-            gobject.idle_add(self._restore_selection, selection)
-
-    def _restore_selection(self, selection):
-        iters = [self.day_text_buffer.get_iter_at_offset(offset)
-                 for offset in selection]
-        self.day_text_buffer.select_range(*iters)
+    def show_template(self, title, text):
+        buf = self._get_buffer(('template', title), text)
+        self.replace_buffer(buf)
         self.day_text_view.grab_focus()
 
 
-class NewEntryDialog(object):
+class NewEntryDialog:
     def __init__(self, main_frame):
         dialog = main_frame.builder.get_object('new_entry_dialog')
         self.dialog = dialog
@@ -692,17 +802,19 @@ class NewEntryDialog(object):
 
         self.main_frame = main_frame
         self.journal = self.main_frame.journal
-        self.categories_combo_box = CustomComboBoxEntry(main_frame.builder.get_object('categories_combo_box'))
-        self.new_entry_combo_box = CustomComboBoxEntry(main_frame.builder.get_object('entry_combo_box'))
+        self.categories_combo_box = CustomComboBoxEntry(
+            main_frame.builder.get_object('categories_combo_box'))
+        self.new_entry_combo_box = CustomComboBoxEntry(
+            main_frame.builder.get_object('entry_combo_box'))
 
         # Let the user finish a new category entry by hitting ENTER
         def respond(widget):
             if self._text_entered():
-                self.dialog.response(gtk.RESPONSE_OK)
+                self.dialog.response(Gtk.ResponseType.OK)
         self.new_entry_combo_box.entry.connect('activate', respond)
         self.categories_combo_box.entry.connect('activate', respond)
 
-        self.categories_combo_box.connect('changed', self.on_category_changed)
+        self.categories_combo_box.combo_box.connect('changed', self.on_category_changed)
 
     def on_category_changed(self, widget):
         '''Show old entries in ComboBox when a new category is selected'''
@@ -711,7 +823,7 @@ class NewEntryDialog(object):
         self.new_entry_combo_box.set_entries(old_entries)
 
         # only make the entry submittable, if text has been entered
-        self.dialog.set_response_sensitive(gtk.RESPONSE_OK, self._text_entered())
+        self.dialog.set_response_sensitive(Gtk.ResponseType.OK, self._text_entered())
 
     def _text_entered(self):
         return bool(self.categories_combo_box.get_active_text())
@@ -737,7 +849,7 @@ class NewEntryDialog(object):
         response = self.dialog.run()
         self.dialog.hide()
 
-        if not response == gtk.RESPONSE_OK:
+        if not response == Gtk.ResponseType.OK:
             return
 
         category_name = self.categories_combo_box.get_active_text()
@@ -752,19 +864,15 @@ class NewEntryDialog(object):
         self.main_frame.cloud.update()
 
 
-class Statusbar(object):
+class Statusbar:
     def __init__(self, statusbar):
         self.statusbar = statusbar
-        self.context_id = self.statusbar.get_context_id('RedNotebook')
+        self.context_id = self.statusbar.get_context_id(info.program_name)
         self.last_message_id = None
         self.timespan = 10
 
     def remove_message(self):
-        if hasattr(self.statusbar, 'remove_message'):
-            self.statusbar.remove_message(self.context_id, self.last_message_id)
-        else:
-            # Deprecated
-            self.statusbar.remove(self.context_id, self.last_message_id)
+        self.statusbar.remove(self.context_id, self.last_message_id)
 
     def _show_text(self, text, countdown=True):
         if self.last_message_id is not None:
@@ -782,52 +890,46 @@ class Statusbar(object):
 
     def start_countdown(self):
         self.time_left = self.timespan
-        self.countdown = gobject.timeout_add(1000, self.count_down)
+        self.countdown = GObject.timeout_add(1000, self.count_down)
 
     def count_down(self):
         self.time_left -= 1
         if self.time_left <= 0:
-            gobject.source_remove(self.countdown)
+            GObject.source_remove(self.countdown)
             self._show_text('', countdown=False)
         return True
 
 
-class Calendar(object):
+class MainCalendar:
     def __init__(self, journal, calendar):
         self.journal = journal
         self.calendar = calendar
 
-        week_numbers = self.journal.config.read('weekNumbers')
-        if week_numbers:
+        if self.journal.config.read('weekNumbers'):
             calendar.set_property('show-week-numbers', True)
 
         self.date_listener = self.calendar.connect('day-selected', self.on_day_selected)
 
-    def on_day_selected(self, cal):
+    def on_day_selected(self, _cal):
         self.journal.change_date(self.get_date())
 
     def set_date(self, date):
-        '''
-        A date check makes no sense here since it is normal that a new month is
-        set here that will contain the day
-        '''
-        # Probably useless
         if date == self.get_date():
             return
 
-        # We do not want to listen to this programmatic date change
+        # We do not want to listen to programmatic date changes.
         self.calendar.handler_block(self.date_listener)
 
-        # We need to set the day temporarily to a day that is present in all months
+        # We need to set the day temporarily to a day that is present in all months.
         self.calendar.select_day(1)
 
-        # PyGTK calendars show months in range [0,11]
+        # GTK shows months in range [0,11].
         self.calendar.select_month(date.month - 1, date.year)
 
-        # Select the day after the month and year have been set
+        # Select the day after the month and year have been set.
         self.calendar.select_day(date.day)
 
-        # We want to listen to manual date changes
+        # We want to listen to manual date changes.
         self.calendar.handler_unblock(self.date_listener)
 
     def get_date(self):
@@ -861,7 +963,7 @@ class Calendar(object):
         in the calendar. This behaviour is necessary since we use the calendar
         both for navigating and showing the current date.
         '''
-        cal_year, cal_month, cal_day = self.calendar.get_date()
+        cal_year, cal_month, _cal_day = self.calendar.get_date()
         cal_month += 1
         if day_number not in range(1, dates.get_number_of_days(cal_year, cal_month) + 1):
             logging.debug(
