@@ -19,7 +19,7 @@
 
 import os
 
-import gtk
+from gi.repository import Gtk
 
 from rednotebook.util import filesystem
 from rednotebook.util import dates
@@ -67,7 +67,7 @@ You can link to almost everything:
 
 - **links to files on your computer:** [filename.txt ""/path/to/filename.txt""]
 - **links to directories:** [directory name ""/path/to/directory/""]
-- **links to websites:** [RedNotebook Homepage ""http://rednotebook.sourceforge.net""]
+- **links to websites:** [RedNotebook Homepage ""https://rednotebook.sourceforge.io""]
 
 
 As you see, **bullet lists** are also available. As always you have to add two \
@@ -174,7 +174,33 @@ personal = _('''\
 ''')
 
 
-class TemplateManager(object):
+class TemplateInfo(Gtk.InfoBar):
+    def __init__(self):
+        Gtk.InfoBar.__init__(self)
+        self.set_message_type(Gtk.MessageType.INFO)
+
+        title_label = Gtk.Label()
+        title_label.set_markup('<b>{}</b>'.format(_('Template mode')))
+        title_label.set_alignment(0., 0.5)
+
+        msg_label = Gtk.Label()
+        msg_label.set_markup(_('You are currently editing a template.'))
+        msg_label.set_alignment(0., 0.5)
+
+        vbox = Gtk.VBox(spacing=5)
+        vbox.pack_start(title_label, False, False, 0)
+        vbox.pack_start(msg_label, False, False, 0)
+
+        image = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_INFO, Gtk.IconSize.DIALOG)
+
+        content = self.get_content_area()
+        content.pack_start(image, False, False, 0)
+        content.pack_start(vbox, False, False, 0)
+
+        self.show_all()
+
+
+class TemplateManager:
     def __init__(self, main_window):
         self.main_window = main_window
 
@@ -187,10 +213,12 @@ class TemplateManager(object):
         self.merge_id = None
         self.actiongroup = None
         self.tmp_title = None
-        self.tmp_parts = None
+        self.selection_to_replace = None
 
-        style = self.main_window.day_text_field.day_text_view.get_style()
-        self.default_base_color = style.base[gtk.STATE_NORMAL]
+        self._template_mode_info_bar = TemplateInfo()
+        self._template_mode_info_bar.hide()
+        self.main_window.text_vbox.pack_start(self._template_mode_info_bar, False, False, 0)
+        self.main_window.text_vbox.reorder_child(self._template_mode_info_bar, 1)
 
     def set_template_menu_sensitive(self, sensitive):
         if self.tmp_title:
@@ -202,10 +230,8 @@ class TemplateManager(object):
         self.main_window.calendar.calendar.set_sensitive(sensitive)
         journal_menu_item = self.main_window.uimanager.get_widget('/MainMenuBar/Journal')
         for child in journal_menu_item.get_submenu().get_children():
-            if isinstance(child, gtk.MenuItem):
-                action = child.get_action()
-                if action:
-                    action.set_sensitive(sensitive)
+            if isinstance(child, Gtk.MenuItem):
+                child.set_sensitive(sensitive)
         self.set_template_menu_sensitive(sensitive)
         for widget in [
                 self.main_window.back_one_day_button,
@@ -214,56 +240,53 @@ class TemplateManager(object):
                 self.main_window.search_tree_view,
                 self.main_window.search_box.entry,
                 self.main_window.cloud,
-                self.main_window.uimanager.get_widget('/MainMenuBar/Edit/Find').get_action()]:
+                self.main_window.uimanager.get_widget('/MainMenuBar/Edit/Find')]:
             widget.set_sensitive(sensitive)
 
-    def enter_template_mode(self, title, parts):
+    def enter_template_mode(self, title, selection):
         # Save the templates title and the day's text.
         self.tmp_title = title
-        self.tmp_parts = parts
+        self.selection_to_replace = selection
         self.main_window.template_bar.show()
         text = self.get_text(title)
-        self.main_window.undo_redo_manager.set_stack(title)
-        self.main_window.day_text_field.set_text(text, undoing=True)
-        light_yellow = gtk.gdk.Color(1., 1., 190 / 255., 0)
-        self.main_window.day_text_field.day_text_view.modify_base(gtk.STATE_NORMAL, light_yellow)
+        self.main_window.day_text_field.show_template(title, text)
+        self._template_mode_info_bar.show()
         self._set_widgets_sensitive(False)
 
     def exit_template_mode(self):
         self.tmp_title = None
-        self.tmp_parts = None
+        self.selection_to_replace = None
         self.main_window.template_bar.hide()
         if self.main_window.preview_mode:
             self.main_window.change_mode(preview=False)
         self.main_window.day_text_field.day_text_view.grab_focus()
-        self.main_window.day_text_field.day_text_view.modify_base(gtk.STATE_NORMAL,
-                                                                  self.default_base_color)
+        self._template_mode_info_bar.hide()
         self._set_widgets_sensitive(True)
 
-    def _reset_undo_stack(self):
-        self.main_window.undo_redo_manager.set_stack(self.main_window.day.date)
-
     def edit(self, title):
-        parts = self.main_window.day_text_field.get_text_parts()
-        self.enter_template_mode(title, parts)
+        selection = self.main_window.day_text_field.get_selection_bounds()
+        self.enter_template_mode(title, selection)
 
     def _replace_macros(self, text):
         # convert every "$date$" to the current date
         config = self.main_window.journal.config
-        format_string = config.read('dateTimeString', '%A, %x %X')
+        format_string = config.read('dateTimeString')
         date_string = dates.format_date(format_string)
-        text = text.replace(u'$date$', date_string)
+        text = text.replace('$date$', date_string)
         return text
 
     def on_save_insert(self, button):
         self.on_save(None)
         template = self.main_window.day_text_field.get_text()
         template = self._replace_macros(template)
-        p1, p2, p3 = self.tmp_parts
-        self._reset_undo_stack()
-        # Force addition of an undo item.
-        self.main_window.day_text_field.set_text(p1 + p2 + p3, undoing=True)
-        self.main_window.day_text_field.set_text(p1 + template + p3)
+
+        self.main_window.day_text_field.show_day(self.main_window.day)
+        buf = self.main_window.day_text_field.day_text_buffer
+        buf.begin_user_action()
+        buf.select_range(*self.selection_to_replace)
+        buf.delete_selection(interactive=False, default_editable=True)
+        buf.insert_at_cursor(template)
+        buf.end_user_action()
         self.exit_template_mode()
 
     def on_save(self, button):
@@ -271,41 +294,39 @@ class TemplateManager(object):
         filesystem.write_file(self.get_path(self.tmp_title), template)
 
     def on_close(self, button):
-        self._reset_undo_stack()
-        self.main_window.day_text_field.set_text(''.join(self.tmp_parts), undoing=True)
+        self.main_window.day_text_field.show_day(self.main_window.day)
         self.exit_template_mode()
 
     def on_new_template(self, action):
-        dialog = gtk.Dialog(_('Choose Template Name'))
+        dialog = Gtk.Dialog(_('Choose Template Name'))
         dialog.set_transient_for(self.main_window.main_frame)
-        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
-        dialog.set_response_sensitive(gtk.RESPONSE_OK, False)
+        dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dialog.add_button(Gtk.STOCK_OK, Gtk.ResponseType.OK)
+        dialog.set_response_sensitive(Gtk.ResponseType.OK, False)
 
         # Let user finish by hitting ENTER.
         def respond(widget):
-            dialog.response(gtk.RESPONSE_OK)
+            dialog.response(Gtk.ResponseType.OK)
 
         def on_text_changed(entry):
-            dialog.set_response_sensitive(gtk.RESPONSE_OK,
-                                          bool(entry.get_text().decode('UTF-8')))
+            dialog.set_response_sensitive(Gtk.ResponseType.OK, bool(entry.get_text()))
 
-        entry = gtk.Entry()
+        entry = Gtk.Entry()
         entry.connect('activate', respond)
         # Only allow closing dialog when text is entered.
         entry.connect('changed', on_text_changed)
         entry.set_size_request(300, -1)
-        dialog.get_content_area().pack_start(entry)
+        dialog.get_content_area().pack_start(entry, True, True, 0)
         dialog.show_all()
         response = dialog.run()
         dialog.hide()
 
-        if response == gtk.RESPONSE_OK:
-            title = entry.get_text().decode('UTF-8')
-            parts = self.main_window.day_text_field.get_text_parts()
+        if response == Gtk.ResponseType.OK:
+            title = entry.get_text()
+            selection = self.main_window.day_text_field.get_selection_bounds()
             path = self.get_path(title)
             filesystem.make_file(path, example_text)
-            self.enter_template_mode(title, parts)
+            self.enter_template_mode(title, selection)
 
     def _get_weekday_number(self):
         return self.main_window.journal.date.weekday() + 1
@@ -335,7 +356,7 @@ class TemplateManager(object):
 
     def get_menu(self):
         '''
-        See http://www.pygtk.org/pygtk2tutorial/sec-UIManager.html for help
+        See http://www.pyGtk.org/pygtk2tutorial/sec-UIManager.html for help
         A popup menu cannot show accelerators (HIG).
         '''
         files = self.get_available_template_files()
@@ -349,7 +370,7 @@ class TemplateManager(object):
         actions_xml = ''.join('<menuitem action="Edit%s"/>' %
                               self._escape_template_name(title)
                               for title in sorted(titles)
-                              if title not in map(str, range(1, 8)))
+                              if title not in '1234567')
 
         uimanager = self.main_window.uimanager
 
@@ -357,7 +378,7 @@ class TemplateManager(object):
             uimanager.remove_action_group(self.actiongroup)
 
         # Create an ActionGroup
-        self.actiongroup = gtk.ActionGroup('TemplateActionGroup')
+        self.actiongroup = Gtk.ActionGroup('TemplateActionGroup')
 
         # Create actions
         actions = []
@@ -372,11 +393,11 @@ class TemplateManager(object):
                            get_edit_function(title))
             actions.append(edit_action)
 
-        actions.append(('EditWeekday', gtk.STOCK_HOME,
+        actions.append(('EditWeekday', Gtk.STOCK_HOME,
                         _("This Weekday's Template"), None, None,
                         lambda button: self.edit('Weekday')))
 
-        actions.append(('NewTemplate', gtk.STOCK_NEW, _('Create New Template'),
+        actions.append(('NewTemplate', Gtk.STOCK_NEW, _('Create New Template'),
                         None, None, self.on_new_template))
 
         self.actiongroup.add_actions(actions)
